<?php

namespace Drupal\mercury_editor_content_moderation;

use Drupal\Core\Url;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Ajax\RedirectCommand;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\RevisionLogInterface;
use Drupal\Core\Entity\RevisionableInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\layout_paragraphs\LayoutParagraphsLayout;
use Drupal\Core\Entity\ContentEntityStorageInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\content_moderation\ModerationInformationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\content_moderation\StateTransitionValidationInterface;
use Drupal\layout_paragraphs\Access\LayoutParagraphsBuilderAccess;
use Drupal\Tests\layout_paragraphs\Functional\LayoutParagraphsTest;
use Drupal\layout_paragraphs\LayoutParagraphsLayoutTempstoreRepository;

/**
 * Adds moderation state manageement to the layout paragraphs builder form.
 */
class ContentModerationFormElements implements ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The moderation information service.
   *
   * @var \Drupal\content_moderation\ModerationInformationInterface
   */
  protected $moderationInformation;

  /**
   * The moderation state transition validation service.
   *
   * @var \Drupal\content_moderation\StateTransitionValidation
   */
  protected $validation;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity repository service.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * The Layout Paragraphs tempstore service.
   *
   * @var \Drupal\layout_paragraphs\LayoutParagraphsLayoutTempstoreRepository
   */
  protected $tempstore;

  /**
   * The layout paragraphs builder access object.
   *
   * @var \Drupal\layout_paragraphs\Access\LayoutParagraphsBuilderAccess
   */
  protected $layoutParagraphsBuilderAccess;

  /**
   * QuickDraftForm constructor.
   *
   * @param \Drupal\content_moderation\ModerationInformationInterface $moderation_info
   *   The moderation information service.
   * @param \Drupal\content_moderation\StateTransitionValidationInterface $validation
   *   The moderation state transition validation service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository.
   */
  public function __construct(
    AccountProxyInterface $current_user,
    ConfigFactoryInterface $config_factory,
    ModerationInformationInterface $moderation_info,
    StateTransitionValidationInterface $validation,
    EntityTypeManagerInterface $entity_type_manager,
    EntityRepositoryInterface $entity_repository,
    LayoutParagraphsLayoutTempstoreRepository $tempstore,
    LayoutParagraphsBuilderAccess $layout_paragraphs_builder_access
  ) {
    $this->currentUser = $current_user;
    $this->configFactory = $config_factory;
    $this->moderationInformation = $moderation_info;
    $this->validation = $validation;
    $this->entityTypeManager = $entity_type_manager;
    $this->entityRepository = $entity_repository;
    $this->tempstore = $tempstore;
    $this->layoutParagraphsBuilderAccess = $layout_paragraphs_builder_access;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
      $container->get('config.factory'),
      $container->get('content_moderation.moderation_information'),
      $container->get('content_moderation.state_transition_validation'),
      $container->get('entity_type.manager'),
      $container->get('entity.repository'),
      $container->get('layout_paragraphs.tempstore_repository'),
      $container->get('layout_paragraphs.builder_access')
    );
  }

  /**
   * Builds the content moderation form elements.
   *
   * Uses settings form moderation_sidebar.module.
   * @see \Drupal\moderation_sidebar\Form\QuickTransitionForm
   *
   * @param \Drupal\layout_paragraphs\LayoutParagraphsLayout $layout
   *   A Layout Paragraphs Layout object.
   *
   * @return array
   *   The elements array.
   */
  public function formElements(LayoutParagraphsLayout $layout) {
    $form = [];
    $entity = $layout->getEntity();

    $transitions = $this->validation->getValidTransitions($entity, $this->currentUser);
    $workflow = $this->moderationInformation->getWorkFlowForEntity($entity);
    $disabled_transitions = $this->configFactory
      ->getEditable('moderation_sidebar.settings')
      ->get('workflows.' . $workflow->id() . '_workflow.disabled_transitions');

    /** @var \Drupal\content_moderation\Entity\ContentModerationStateInterface $current_state */
    $current_state = $this->getModerationState($entity);

    $is_transition_enabled = FALSE;
    foreach ($transitions as $transition) {
      // Exclude disabled transitions.
      if (empty($disabled_transitions[$transition->id()])) {
        // Use "Save" for self-transition.
        $label = $transition->to()->id() == $current_state->id() ? $this->t('Save') : $transition->label();
        $form[$transition->id()] = [
          '#type' => 'submit',
          '#ajax' => [
            'callback' => [$this, 'ajaxCallback'],
          ],
          '#submit' => [[$this, 'submitForm']],
          '#layout_paragraphs_layout_id' => $layout->id(),
          '#id' => $transition->id(),
          '#name' => $transition->id(),
          '#value' => $label,
          '#attributes' => [
            'class' => ['button--primary'],
          ],
        ];
        // @todo Revisit custom moderation messages.
        // $is_transition_enabled = TRUE;
      }
    }

    // Show only, if at least one transition is enabled.
    if ($is_transition_enabled) {
      $form['revision_log_toggle'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Use custom log message'),
        '#default_value' => FALSE,
        '#attributes' => [
          'class' => ['moderation-sidebar-revision-log-toggle'],
        ],
      ];
      $form['revision_log'] = [
        '#type' => 'textarea',
        '#placeholder' => $this->t('Briefly describe this state change.'),
        '#attributes' => [
          'class' => ['moderation-sidebar-revision-log'],
        ],
        '#states' => [
          'visible' => [
            ':input[name="revision_log_toggle"]' => [
              'checked' => TRUE,
            ],
          ],
        ],
      ];
    }
    return $form;
  }

  /**
   * Submit handler for moderation buttons.
   *
   * Saves the layout and transitions the parent entity to the correct state.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $element = $form_state->getTriggeringElement();
    $layout = $this->tempstore->getWithStorageKey($element['#layout_paragraphs_layout_id']);
    $entity = $layout->getEntity();
    $field_name = $layout->getFieldName();
    $entity->$field_name = $layout->getParagraphsReferenceField();
    /** @var \Drupal\content_moderation\Entity\ContentModerationStateInterface[] $transitions */
    $transitions = $this->validation->getValidTransitions($entity, $this->currentUser);
    /** @var \Drupal\content_moderation\ContentModerationState $state */
    $state = $transitions[$element['#id']]->to();
    $state_id = $state->id();
    if ($form_state->getValue('revision_log_toggle')) {
      $revision_log = $form_state->getValue('revision_log');
    }
    else {
      $revision_log = $this->t('Used the Layout Paragraphs frontend builder and set moderation state to "@state".', ['@state' => $state->label()]);
    }
    /** @var \Drupal\Core\Entity\ContentEntityInterface $revision */
    $revision = $this->prepareNewRevision($entity, $revision_log);
    $revision->set('moderation_state', $state_id);
    $revision->save();
    $layout->setParagraphsReferenceField($revision->$field_name);
    $this->tempstore->set($layout);
  }

  /**
   * The form ajax callback.
   *
   * Returns the rendered layout paragraphs builder or redirects to the
   * main view tab if the current user doesn't have access.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function ajaxCallback(array &$form, FormStateInterface $form_state) {
    $element = $form_state->getTriggeringElement();
    $layout = $this->tempstore->getWithStorageKey($element['#layout_paragraphs_layout_id']);
    $response = new AjaxResponse();
    $response->addCommand(new ReplaceCommand('[data-lpb-form-id="' . $form['#attributes']['data-lpb-form-id'] . '"]', $form));
    if ($this->layoutParagraphsBuilderAccess->access($this->currentUser, $layout)->isForbidden()) {
      $entity = $layout->getEntity();
      $url = $entity->toUrl();
      $command = new RedirectCommand($url->toString());
      $response->addCommand($command);
    }
    return $response;
  }

  /**
   * Gets the Moderation State of a given Entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   An entity.
   *
   * @return \Drupal\workflows\StateInterface
   *   The moderation state for the given entity.
   */
  protected function getModerationState(ContentEntityInterface $entity) {
    $state_id = $entity->moderation_state->get(0)->getValue()['value'];
    $workflow = $this->moderationInformation->getWorkFlowForEntity($entity);
    return $workflow->getTypePlugin()->getState($state_id);
  }

  /**
   * Prepares a new revision of a given entity, if applicable.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   An entity.
   * @param string|\Drupal\Core\StringTranslation\TranslatableMarkup $message
   *   A revision log message to set.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   The moderation state for the given entity.
   */
  protected function prepareNewRevision(EntityInterface $entity, $message) {
    $storage = $this->entityTypeManager->getStorage($entity->getEntityTypeId());
    if ($storage instanceof ContentEntityStorageInterface) {
      $revision = $storage->createRevision($entity);
      if ($revision instanceof RevisionLogInterface) {
        $revision->setRevisionLogMessage($message);
        $revision->setRevisionCreationTime(\Drupal::time()->getRequestTime());
        $revision->setRevisionUserId($this->currentUser->id());
      }
      return $revision;
    }
    return $entity;
  }

}
