/**
* DO NOT EDIT THIS FILE.
* See the following change record for more information,
* https://www.drupal.org/node/2815083
* @preserve
**/

(function ($, Drupal, drupalSettings) {
  drupalSettings.mercuryDialog = {
    autoOpen: true,
    autoResize: false,
    dialogClass: '',
    buttonClass: 'button',
    buttonPrimaryClass: 'button--primary',
    close: function close(event) {
      Drupal.mercuryDialog(event.target).close();
      Drupal.detachBehaviors(event.target, null, 'unload');
    }
  };

  Drupal.mercuryDialog = function (element, options) {
    var undef;
    var $element = $(element);
    var dialogElement;
    var dialog = {
      open: false,
      returnValue: undef,
    };

    /**
     * ResizeObserver callback that runs when the shadow dialog element resizes.
     *
     * @param {ResizeObserverEntry[]} entries
     */
    function onDockResize(entries) {
      for (const entry of entries) {
        const element = entry.target;
        const dockDirection = element.getAttribute('data-dock');
        if (!element.open || !dockDirection || dockDirection == 'none') {
          return;
        }

        // Persist the width if the is docked left or right.
        if (['right', 'left'].includes(dockDirection)) {
          const width = Math.floor(entry.contentBoxSize[0].inlineSize);
          localStorage.setItem('mercury-dialog-dock-width', width);
        }
        // Persist the height if the is docked top or bottom.
        if (['top', 'bottom'].includes(dockDirection)) {
          const height = Math.floor(entry.contentBoxSize[0].blockSize);
          localStorage.setItem('mercury-dialog-dock-height', height);
        }
      }
    }

    const dockObserver = new ResizeObserver(onDockResize);

    /**
     * MutationObserver callback that will start the resize observer when the dialog opens.
     *
     * @param {MutationRecord[]} entries
     */
    function onDialogMutate(records) {
      // @todo: We may only want to attach this if the dialog is transitioning to an open state.
      dockObserver.observe(dialogElement.shadowRoot.querySelector('dialog'));
    }

    const dialogMutationObserver = new MutationObserver(onDialogMutate);

    /**
     * Converts a numeric only value to a px based CSS value.
     * @param {*} value
     * @returns {string}
     *   A CSS length value.
     */
    function getCSSLength(value) {
      if (typeof value === 'undefined' || !/^\d+$/.test(value)) {
        return value;
      }
      return `${value}px`;
    }

    /**
     * Apply command response options to the <mercury-dialog> element
     *
     * @param {Element} dialogElement
     * @param {Object} options
     */
    function applyOptions(dialogElement, options) {
      const attributeOptions = [
        'title',
        'modal',
        'dock',
        'push',
        'resizable',
      ];

      let { width, height } = options;

      // Set these options as HTML attributes on the element.
      attributeOptions.forEach(option => {
        if (typeof options[option] !== 'undefined') {
          dialogElement.setAttribute(option, options[option]);
        }
      });

      if (options.dock) {
        const savedWidth = localStorage.getItem('mercury-dialog-dock-width');
        const savedHeight = localStorage.getItem('mercury-dialog-dock-height');

        if (savedWidth && ['right', 'left'].includes(options.dock)) {
          dialogElement.setAttribute('width', savedWidth);
        }
        if (savedHeight && ['top', 'bottom'].includes(options.dock)) {
          dialogElement.setAttribute('height', savedHeight);
        }
      }

      if (options.dialogClass) {
        dialogElement.classList = options.dialogClass;
      }

      if (typeof width !== 'undefined') {
        dialogElement.style.setProperty("--me-dialog-width-default", getCSSLength(width));
      }

      if (typeof height !== 'undefined') {
        dialogElement.style.setProperty("--me-dialog-height-default", getCSSLength(height));
      }

      _createButtonPane();

      return dialogElement;
    }

    /**
     * Determines which element to append the dialog to. Defaults to the <body>
     * @returns Element
     */
    function _appendTo() {
      var element = options.appendTo;
      if ( element && ( element.jquery || element.nodeType ) ) {
        return $( element );
      }
      return $(document).find( element || "body" ).eq( 0 );
    };

    /**
     * Create a button pane similar to jQuery UI dialog.
     */
    function _createButtonPane() {
      dialog.uiDialogButtonPane = $('<div class="me-dialog__buttonpane">');
      _createButtons();
    };

    /**
     * Populate the button pane with buttons similar to jQuery UI dialog.
     */
    function _createButtons() {
      const buttons = options.buttons;

      // If we already have a button pane, remove it
      dialog.uiDialogButtonPane.remove();

      if ( $.isEmptyObject( buttons ) || ( Array.isArray( buttons ) && !buttons.length ) ) {
        return;
      }

      $.each( buttons, function( name, props ) {
        var click, buttonOptions;
        props = typeof props === "function" ?
          { click: props, text: name } :
          props;

        // Default to a non-submitting button
        props = $.extend( { type: "button" }, props );

        // Change the context for the click callback to be the main element
        click = props.click;
        buttonOptions = {
          icon: props.icon,
          iconPosition: props.iconPosition,
          showLabel: props.showLabel,

          // Deprecated options
          icons: props.icons,
          text: props.text
        };

        delete props.click;
        delete props.icon;
        delete props.iconPosition;
        delete props.showLabel;

        // Deprecated options
        delete props.icons;
        if ( typeof props.text === "boolean" ) {
          delete props.text;
        }

        $( "<button></button>", props )
          .button( buttonOptions )
          .appendTo( dialog.uiDialogButtonPane )
          .on( "click", function() {
            click.apply( dialogElement, arguments );
          } );
      } );
      dialog.uiDialogButtonPane.appendTo( dialogElement );
    };

    function init(settings) {
      // Wrap the element in a <mercury-dialog> if it isn't already.
      if (element.tagName !== 'MERCURY-DIALOG') {
        const wrapper = $('<mercury-dialog>').append($element).appendTo(_appendTo());
        dialogElement = wrapper[0];
      } else {
        dialogElement = element;
      }

      applyOptions(dialogElement, settings);
    }

    /**
     * Initializes and opens a dialog element.
     * @param {Object} settings
     *   Dialog settings mimicing jQuery UI for compatability.
     */
    function openDialog(settings) {
      settings = {...drupalSettings.dialog, ...options, ...settings};
      $(window).trigger('mercuryDialog:beforecreate', [dialog, $element, settings]);
      init(settings);
      dialogElement[settings.modal ? 'showModal' : 'show']();
      $(window).trigger('mercuryDialog:aftercreate', [dialog, $element, settings]);

      // Add a mutation observer to the dialog element.
      dialogMutationObserver.observe(dialogElement, { childList: true, attributes: true });

      dialogElement.addEventListener('close', function () {
        dialogElement.remove();
      });
    }

    function closeDialog(value) {
      $(window).trigger('mercuryDialog:beforeclose', [dialog, $element]);

      // Stop observing height and width changes.
      dockObserver.disconnect();
      dialogMutationObserver.disconnect();

      element.close();
      dialog.returnValue = value;
      $(window).trigger('mercuryDialog:afterclose', [dialog, $element]);
    }

    dialog.show = function () {
      openDialog({
        modal: false,
      });
    };

    dialog.showModal = function () {
      openDialog({
        modal: true,
      });
    };

    dialog.close = closeDialog;
    return dialog;
  };

  Drupal.behaviors.mercuryDialog = {
    attach: function(context, settings) {
      // Provide a known 'drupal-mercury-dialog' DOM element for Drupal-based modal
      // dialogs. Non-modal dialogs are responsible for creating their own
      // elements, since there can be multiple non-modal dialogs at a time.

      if (!$('#drupal-mercury-dialog').length) {
        // Add 'ui-front' jQuery UI class so jQuery UI widgets like autocomplete
        // sit on top of dialogs. For more information see
        // http://api.jqueryui.com/theming/stacking-elements/.
        // @todo: .ui-front just sets a z-index of 100 which is not high enough
        // to overlay Gin's Toolbar.
        $('<mercury-dialog id="drupal-mercury-dialog"></mercury-dialog>')
          .appendTo('body');
      }
    },

    openMercuryDialog: function (ajax, response, status) {
      if (!response.selector) {
        return false;
      }
      let $dialog = $(response.selector);
      if (!$dialog.length) {
        // Create the element if needed.
        $dialog = $(
          `<mercury-dialog id="${response.selector.replace(
            /^#/,
            '',
          )}" class="ui-front"></mercury-dialog>`,
        ).appendTo('body');
      }
      // Set up the wrapper, if there isn't one.
      if (!ajax.wrapper) {
        ajax.wrapper = $dialog.attr('id');
      }

      // Use the ajax.js insert command to populate the dialog contents.
      response.command = 'insert';
      response.method = 'html';
      ajax.commands.insert(ajax, response, status);

      // // Move the buttons to the jQuery UI dialog buttons area.
      // if (!response.dialogOptions.buttons) {
      //   response.dialogOptions.drupalAutoButtons = true;
      //   response.dialogOptions.buttons =
      //     Drupal.behaviors.mercuryDialog.prepareDialogButtons($dialog);
      // }

      // // Bind dialogButtonsChange.
      // $dialog.on('dialogButtonsChange', () => {
      //   const buttons = Drupal.behaviors.mercuryDialog.prepareDialogButtons($dialog);
      //   $dialog.dialog('option', 'buttons', buttons);
      // });

      // Open the dialog itself.
      response.dialogOptions = response.dialogOptions || {};
      const dialog = Drupal.dialog($dialog.get(0), response.dialogOptions);
      if (response.dialogOptions.modal) {
        dialog.showModal();
      } else {
        dialog.show();
      }

      // Add the standard Drupal class for buttons for style consistency.
      $dialog.parent().find('.ui-dialog-buttonset').addClass('form-actions');
    },

    getCSSLength: function(value) {
      if (typeof value === 'undefined') {
        return value;
      }

      const numeric = parseFloat(value);

      // @todo: This assumes a numeric value is meant to be pixels which doesn't
      // account for values with units such as '50%', '16em' or '2rem'
      return isNaN(numeric) ? value : `${numeric}px`;
    },

    /**
     * Apply command response options to the <mercury-dialog> element
     *
     * @param {Element} dialog
     * @param {Object} options
     */
    applyOptions: function (dialog, options) {
      if (options.title) {
        dialog.setAttribute('title', options.title);
      }

      if (options.modal) {
        dialog.setAttribute('modal', options.modal);
      }

      if (options.resizable) {
        dialog.settAttribute('resizable', options.resizable);
      }

      if (options.dialogClass) {
        dialog.classList = options.dialogClass;
      }

      if (typeof options.width !== 'undefined') {
        dialog.style.setProperty("--me-dialog-width", Drupal.behaviors.mercuryDialog.getCSSLength(options.width));
      }

      if (typeof options.height !== 'undefined') {
        dialog.style.setProperty("--me-dialog-height", Drupal.behaviors.mercuryDialog.getCSSLength(options.height));
      }

      if (typeof options.buttons !== 'undefined') {
        // Remove existing buttons.
        const existingButtonsContainer = dialog.querySelector('.me-dialog__buttonpane');
        if (existingButtonsContainer) {
          existingButtonsContainer.remove();
        }
        if (options.buttons.length) {
          // Add in the new ones.
          const buttonsContainer = document.createElement('div');
          buttonsContainer.classList = 'me-dialog__buttonpane';
          buttonsContainer.setAttribute('slot', 'footer');
          dialog.appendChild(buttonsContainer);
          options.buttons.forEach((props) => {
            const button = document.createElement('button');
            button.classList = props.class;
            button.classList.add('button');
            button.appendChild(document.createTextNode(props.text));
            button.addEventListener('click', props.click);
            buttonsContainer.appendChild(button);
          });
        }
      }
    },

    prepareDialogButtons: function prepareDialogButtons($dialog) {
      var buttons = [];
      var $buttons = $dialog.find('.me-form-actions input[type=submit], .me-form-actions a.button, .me-form-actions a.action-link');
      $buttons.each(function () {
        var $originalButton = $(this).css({
          display: 'none'
        });
        buttons.push({
          text: $originalButton.html() || $originalButton.attr('value'),
          class: $originalButton.attr('class'),
          click: function click(e) {
            if ($originalButton.is('a')) {
              $originalButton[0].click();
            } else {
              $originalButton.trigger('mousedown').trigger('mouseup').trigger('click');
              e.preventDefault();
            }
          }
        });
      });
      return buttons;
    }
  }

})(jQuery, Drupal, drupalSettings);
