(function () {
  'use strict';

  (function (Drupal, $, once) {
    /**
     * Gets the mercury dialog library if available, otherwise defaults to Drupal's jQuery UI dialog.
     *
     * @returns {function}
     */
    function getDialogLibrary() {
      return typeof Drupal.mercuryDialog === 'function' ? Drupal.mercuryDialog : Drupal.dialog;
    }

    const genRanHex = size => [...Array(size)].map(() => Math.floor(Math.random() * 16).toString(16)).join('');

    /**
     * Opens the link in an iframe within a dialog element.
     *
     * @param {HTMLAnchorElement} a
     *   An anchor tag with an href attribute to the url that should be loaded within the iframe.
     */
    function openIframe(options) {
      const dialogLibrary = getDialogLibrary();
      const iframeDialog = document.createElement('div');
      const href = options.href + ( options.href.indexOf('?') == -1 ? '?_mercury_editor_iframe=true' : '&_mercury_editor_iframe=true' );
      delete options.href;

      // Close any open dialogs with the exact same iframe.
      const existingIframe = document.querySelector(`iframe[src="${href}"]`);
      if (existingIframe) {
        const existingIframeDialog = existingIframe.closest('mercury-dialog');
        if (existingIframeDialog) {
          existingIframeDialog.remove();
        }
      }

      iframeDialog.id = 'lpb-dialog-iframe-' + genRanHex(6);
      iframeDialog.classList.add('ui-front');
      iframeDialog.innerHTML = `<div class="me-ajax-progress"><div class="spinner"></div></div>
      <iframe src='${href}' id='mercury-editor-iframe-${genRanHex(6)}' width='100%' height='100%' frameborder='0' class='me-iframe' />`;
      Drupal.behaviors.mercuryEditorIframe.attach(iframeDialog);
      const defaultSettings = {
        width: '90%',
        height: typeof Drupal.mercuryDialog === 'function' ? 'max-content' : 'auto',
        autoResize: true,
        modal: true,
        drupalAutoButtons: false,
        dialogClass: 'lpb-dialog',
        title: options.title,
      };
      const settings = {
        ...defaultSettings,
        ...(options),
      };
      dialogLibrary(iframeDialog, settings)[settings.modal ? 'showModal' : 'show']();
      document.querySelector(`#${iframeDialog.id} iframe`).addEventListener('load', (e) => {
        const element = e.target.parentNode.querySelector('.me-ajax-progress');
        if (element) {
          element.remove();
        }
      });
    }

    function iFrameLinkClickHandler(a) {
      a.addEventListener('click', (e) => {
        const a = e.currentTarget;
        // If the anchor tag is within an existing dialog, close it.
        const currentDialog = a.closest('[id^="lpb-dialog"]');
        if (currentDialog) {
          getDialogLibrary()(currentDialog).close();
        }
        const options = {
          ...{
            href: a.href,
            title: a.getAttribute('title') || ''
          },
          ...JSON.parse(a.getAttribute('data-dialog-options') || '{}') || {}
        };
        e.stopPropagation();
        e.preventDefault();
        openIframe(options);
        return false;
      });
    }

    Drupal.mercuryEditorIframeRenderComponent = function(url, dialogId) {
      // Send request to refresh the layout.
      Drupal.ajax({ url }).execute();
    };

    Drupal.mercuryEditorIframeOpenDialog = function(href, options) {
      return openIframe({href, ...options});
    };

    Drupal.mercuryEditorIframeCloseDialog = function(href) {
      const dialogElement = document.querySelector(`iframe[src="${href}"]`).parentNode;
      var $dialog = $(dialogElement);
      if ($dialog.length) {
        const dialog = $dialog.get(0);
        // Check to see if this is an instance of jQuery UI dialog.
        if ($dialog.dialog('instance')) {
          getDialogLibrary()(dialog).close();
          $dialog.remove().off('dialogButtonsChange');
        }
        // Check to see if this is an instance of MercuryDialog.
        else if (Drupal.mercuryDialog) {
          const mercuryDialog = dialog.closest('mercury-dialog');
          if (mercuryDialog && typeof mercuryDialog.close === 'function') {
            Drupal.mercuryDialog(mercuryDialog).close();
            mercuryDialog.remove();
          }
        }
      }
    };

    /**
     * Attaches the iframe click handle to .use-iframe links.
     */
     Drupal.behaviors.mercuryEditorIframe = {
      attach: function attach(context) {
        // @todo: Find a better way to handle this. For some reason, this behavior gets
        // attached before the entire layout paragraphs builder markup is inserted into the
        // dom. The results in the edit button on components not working until subsequent calls
        // to Drupal.attachbehaviors. Setting a timeout takes this function call out of the main
        // loop which seems give it enough time to recognize the new elements.
        // This feels obviously janky.
        setTimeout(() => {
          once('mercury-editor-tabs', 'a.use-iframe').forEach(iFrameLinkClickHandler);
        }, 1);
      }
    };

  })(Drupal, jQuery, once);

})();
