(function (Drupal, $, once) {
  const delay = 250;
  let timeout = false;

  /**
   *
   * @param {HTMLElement} context
   */
  function trackIFrames(context) {
    const iframes = context.querySelectorAll('.entity-browser-modal-iframe');

    iframes.forEach(iframe => {
      iframe.onload = function() {
        resizeIframe(iframe);
      }

      // Listen for resizing of the framed body element
      const framedBody = iframe?.contentWindow?.document?.body;
      if (framedBody) {
        framedBody.onresize = function() {
          resizeIframe(iframe);
        }
      }
    });
  }

  /**
   * Resizes an iFrame to match the height of its inner <body> element.
   * @param {HTMLIFrameElement} iframe
   */
  function resizeIframe(iframe) {
    const framedBody = iframe.contentWindow.document.body;
    iframe.style.height = framedBody.offsetHeight + 'px';
  }

  const iframeObserver = new MutationObserver(function(mutations) {
    mutations.forEach(function(mutation) {
      mutation.addedNodes.forEach(function(node) {
        // Only concern ourselves with DOM elements.
        if (node.nodeType === 1) {
          trackIFrames(node);
        }
      });
    });
  });

  // Observe the entire document (including the body)
  iframeObserver.observe(document, {
    childList: true,
    subtree: true,
  });

  function dialogButtons(context) {
    once('me-dialog-buttons', '.me-form-actions', context).forEach((formActions) => {
      const mercuryDialog = window.frameElement.closest('mercury-dialog');
      if (mercuryDialog) {
        const buttons = Drupal.behaviors.mercuryDialog.prepareDialogButtons($(formActions.parentElement));
        Drupal.behaviors.mercuryDialog.applyOptions(window.frameElement.closest('mercury-dialog'), {buttons})
      }
    });
  }

  /**
   * Sets the maximum autogrow height of CKEditor based on the size of the parent window.
   */
  function setCKEditorMaxHeight () {
    if (window.CKEDITOR) {
      const drupalOffset = window.parent.Drupal.displace
        ? window.parent.Drupal.displace.offsets.top + window.parent.Drupal.displace.offsets.bottom
        : 0;
      const formButtonsOffset = $('.form-actions').height();
      const maxHeight =  Math.floor(0.5 * (window.parent.innerHeight - drupalOffset - formButtonsOffset));
      window.CKEDITOR.config.autoGrow_maxHeight = maxHeight;
    }
  }

  /**
   * ResizeObserver callback that resizes the parent iframe based on
   * the height of the child document html element.
   *
   * @param {ResizeObserverEntry[]} entries
   */
  function onBodyResize(entries) {
    // Resize the parent iframe based on the html's border box height.
    if (window.frameElement && entries.length) {
      window.frameElement.height = entries[0].borderBoxSize[0].blockSize + 1;
      drupalSettings.dialog.autoResize = false;
    }
  }

  // Observe changes to the <html> dimensions.
  new ResizeObserver(onBodyResize).observe(document.documentElement, { box: 'border-box'});

  /**
   * ResizeObserver callback that resizes the inner document
   * height based on the size of a modal.
   *
   * @param {ResizeObserverEntry[]} entries
   */
  function onModalResize(entries) {
    for (const entry of entries) {
      const element = entry.target.parentElement;
      const contentHeight = element.scrollHeight;
      const body = document.body;
      body.style.height = `${contentHeight}px`;
    }
  }

  const modalResizeObserver = new ResizeObserver(onModalResize);

  /**
   * Observe a jQuery UI Dialog's size after it is created.
   */
  function onDialogAfterCreate(event, dialog, $element) {
    const element = $element[0];
    modalResizeObserver.observe(element);

    const wrapper = element.parentElement;
    document.body.style.height = `${wrapper.getBoundingClientRect().height}px`;
  }

  /**
   * Stop observing a jQuery UI Dialog after it closes
   * and remove any height overrides we added to the <html> element.
   */
  function onDialogAfterClose(event, dialog, $element) {
    const element = $element[0];
    modalResizeObserver.unobserve(element);
    const body = document.body;
    body.style.height = null;
  }

  Drupal.AjaxCommands.prototype.mercuryEditorIframeRefreshLayout = function(ajax, response, status) {
    var { url, dialogId } = response;
    window.parent.Drupal.mercuryEditorIframeRenderComponent(url, dialogId);
  }

  Drupal.AjaxCommands.prototype.mercuryEditorIframeCloseDialog = function(ajax, response, status) {
    window.parent.Drupal.mercuryEditorIframeCloseDialog(window.location.href);
  }

  // Adjust the CKEditor autogrow height when drupal attaches content.
  Drupal.behaviors.mercuryEditorIframeDocument = {
    attach: function attach(context) {
      setCKEditorMaxHeight();
      dialogButtons(context);
      // If link target is not already explicitly set, set it to _parent.
      once('me-anchor-target', 'a', context).forEach((anchorElement) => {
        if (!anchorElement.getAttribute('target')) {
          anchorElement.setAttribute('target', '_parent');
        }
      });
      once('me-iframe-beforeunload', 'html').forEach((htmlElement) => {
        $(window).on("beforeunload", function() {
          Drupal.behaviors.mercuryDialog.applyOptions(window.frameElement.closest('mercury-dialog'), {buttons: []});
        });
      })
    }
  }

  // Adjust the CKEditor autogrow height when the parent window resizes.
  window.parent.addEventListener('resize', function() {
    clearTimeout(timeout);
    timeout = setTimeout(setCKEditorMaxHeight, delay);
  });

  $(window).on('dialog:aftercreate', onDialogAfterCreate);
  $(window).on('dialog:afterclose', onDialogAfterClose);

  trackIFrames(document);

})(Drupal, jQuery, once);
